import redis
try:
    from django.utils.encoding import force_unicode
except ImportError:  # Python 3.*
    from django.utils.encoding import force_text as force_unicode
from django.contrib.sessions.backends.base import SessionBase, CreateError
from redis_sessions import settings


# Avoid new redis connection on each request
if not settings.SESSION_REDIS_URL is None:
    redis_server = redis.StrictRedis.from_url(settings.SESSION_REDIS_URL)
elif settings.SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH is None:
    redis_server = redis.StrictRedis(
        host=settings.SESSION_REDIS_HOST,
        port=settings.SESSION_REDIS_PORT,
        db=settings.SESSION_REDIS_DB,
        password=settings.SESSION_REDIS_PASSWORD
    )
else:
    redis_server = redis.StrictRedis(
        unix_socket_path=settings.SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH,
        db=settings.SESSION_REDIS_DB,
        password=settings.SESSION_REDIS_PASSWORD,
    )


class SessionStore(SessionBase):
    """
    Implements Redis database session store.
    """
    def __init__(self, session_key=None):
        super(SessionStore, self).__init__(session_key)

        self.server = redis_server

    def load(self):
        try:
            session_data = self.server.get(
                self.get_real_stored_key(self._get_or_create_session_key())
            )
            return self.decode(force_unicode(session_data))
        except:
            self.create()
            return {}

    def exists(self, session_key):
        return self.server.exists(self.get_real_stored_key(session_key))

    def create(self):
        while True:
            self._session_key = self._get_new_session_key()

            try:
                self.save(must_create=True)
            except CreateError:
                continue
            self.modified = True
            return

    def save(self, must_create=False):
        session_key = self._get_or_create_session_key()
        if must_create and self.exists(session_key):
            raise CreateError
        data = self.encode(self._get_session(no_load=must_create))
        if redis.VERSION[0] >= 2:
            self.server.setex(
                self.get_real_stored_key(session_key),
                self.get_expiry_age(),
                data
            )
        else:
            self.server.set(
                self.get_real_stored_key(session_key),
                data
            )
            self.server.expire(
                self.get_real_stored_key(session_key),
                self.get_expiry_age()
            )

    def delete(self, session_key=None):
        if session_key is None:
            if self.session_key is None:
                return
            session_key = self.session_key
        try:
            self.server.delete(self.get_real_stored_key(session_key))
        except:
            pass

    def get_real_stored_key(self, session_key):
        if not settings.SESSION_REDIS_PREFIX:
            return session_key
        return '%s:%s' % (settings.SESSION_REDIS_PREFIX, session_key)



import sys
from setuptools import setup


packages = ['redis_sessions']


install_requires = [
    'redis>=2.4.10',
    'django>=1.4'
]

if not "__pypy__" in sys.builtin_module_names:
    install_requires.append('hiredis>=0.1.1')


setup(
    name='django-redis-sessions-fork',
    version='0.5.3',
    description="Redis Session Backend For Django",
    long_description=open('README.rst').read(),
    keywords='django, sessions, redis',
    author='see AUTHORS',
    author_email='hellysmile@gmail.com',
    url='https://github.com/hellysmile/django-redis-sessions-fork',
    license='BSD',
    packages=packages,
    zip_safe=False,
    install_requires=install_requires,
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
    ],
)



import os
from django.conf import settings


SESSION_REDIS_HOST = getattr(settings, 'SESSION_REDIS_HOST', 'localhost')
SESSION_REDIS_PORT = getattr(settings, 'SESSION_REDIS_PORT', 6379)
SESSION_REDIS_DB = getattr(settings, 'SESSION_REDIS_DB', 0)
SESSION_REDIS_PREFIX = getattr(settings, 'SESSION_REDIS_PREFIX', '')
SESSION_REDIS_PASSWORD = getattr(settings, 'SESSION_REDIS_PASSWORD', None)

SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH = getattr(
    settings, 'SESSION_REDIS_UNIX_DOMAIN_SOCKET_PATH', None
)

SESSION_REDIS_URL = getattr(settings, 'SESSION_REDIS_URL', None)

if SESSION_REDIS_URL is None:
    # redis clouds ENV variables
    SESSION_REDIS_ENV_URLS = getattr(
        settings,
        'SESSION_REDIS_ENV_URLS', (
            'REDISTOGO_URL',
            'OPENREDIS_URL',
            'REDISGREEN_URL',
            'MYREDIS_URL'
        )
    )

    for url in SESSION_REDIS_ENV_URLS:
        redis_env_url = os.environ.get(url)
        if redis_env_url:
            SESSION_REDIS_URL = redis_env_url
            break
